// PageEvents.cpp : implementation file
//

#include "stdafx.h"
#include "TrackDemo.h"
#include "PageEvents.h"
#include "DlgShowEvent.h"


// CPageEvents dialog

IMPLEMENT_DYNAMIC(CPageEvents, CPropertyPage)

CPageEvents::CPageEvents()
  : CPropertyPage(CPageEvents::IDD)
{
}

CPageEvents::~CPageEvents()
{
}

void CPageEvents::DoDataExchange(CDataExchange* pDX)
{
  CPropertyPage::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_EVENTS, m_events);
}


BEGIN_MESSAGE_MAP(CPageEvents, CPropertyPage)
  ON_NOTIFY(NM_DBLCLK, IDC_EVENTS, &CPageEvents::OnNMDblclkEvents)
END_MESSAGE_MAP()


// CPageEvents message handlers

BOOL CPageEvents::OnInitDialog() 
{
  // Var
  CRect rect;

  // Default
  CPropertyPage::OnInitDialog();

  m_events.GetClientRect(&rect);
  m_events.InsertColumn(0, _T("Timestamp"), LVCFMT_LEFT, 115);
  m_events.InsertColumn(1, _T("Client"), LVCFMT_LEFT, 70);
  m_events.InsertColumn(2, _T("Event"), LVCFMT_LEFT, rect.Width() - 185);
  m_events.SetExtendedStyle(m_events.GetExtendedStyle() | LVS_EX_FULLROWSELECT);

  // Completed
  return TRUE;
}

void CPageEvents::AddEvent(CTime timestamp, int NodeID, struct FrameEventInfo *eventdata)
{
  // Var
  int index;
  CString text, client;

  // Make event text
  switch (eventdata->event_id) {
    case 0x01: // Position
      text = _T("Position");
      break;

    case 0x02: // Position request
      text = _T("Position request");
      break;

    case 0x03: // Digital input
      index = eventdata->event_data;
      if (index < 0) index = -eventdata->event_data;
      text.Format(_T("Input %d is %s"), index, eventdata->event_data < 0 ? _T("OFF") : _T("ON"));
      break;

    case 0x04: // Geo-fence
      index = eventdata->event_data;
      text.Format(_T("%s geo-fence area"), eventdata->event_data < 0 ? _T("Exited") : _T("Entered"));
      break;

    case 0x05: // Speed
      index = eventdata->event_data;
      text.Format(_T("Velocity is %s threshold"), eventdata->event_data < 0 ? _T("raised abowe") : _T("fallen below"));
      break;

    case 0x06: // External power
      if (eventdata->event_data < 0) text = _T("External power removed");
      else text = _T("External power applied");
      break;

    case 0x07: // Power save mode
      if (eventdata->event_data < 1) text = _T("Requesting power saving mode");
      else {
        text = _T("Power saving mode");
        switch (eventdata->event_data) {
          case 1:   text += _T(", awakened by ignition"); break;
          case 2:   text += _T(", awakened by vibration"); break;
          case 3:   text += _T(", awakened by time expired"); break;
          case 10:  text += _T(" cancelled"); break;
        }
      }
      break;
  };

  // Make node
  client.Format(_T("%d"), NodeID);

  // Var
  index = m_events.InsertItem(m_events.GetItemCount(), timestamp.Format(_T("%Y.%m.%d %H:%M:%S")));
  m_events.SetItemData(index, (DWORD_PTR)eventdata);
  m_events.SetItemText(index, 1, client);
  m_events.SetItemText(index, 2, text);
}

void CPageEvents::OnNMDblclkEvents(NMHDR *pNMHDR, LRESULT *pResult)
{
  // Var
  NMITEMACTIVATE *pItem = (NMITEMACTIVATE*)pNMHDR;
  LVHITTESTINFO info = {0};
  FrameEventInfo *pData;
  *pResult = 0;

  // Get event info
  info.pt = pItem->ptAction;
  m_events.SubItemHitTest(&info);
  pData = (FrameEventInfo*)m_events.GetItemData(info.iItem);

  // Show event information
  CDlgShowEvent dlg;
  dlg.m_name      = m_events.GetItemText(info.iItem, 2);
  dlg.m_serial    = m_events.GetItemText(info.iItem, 1);
  switch (pData->mode) {
    case 0:  dlg.m_mode = _T("No info"); break;
    case 1:  dlg.m_mode = _T("No Fix"); break;
    case 2:  dlg.m_mode = _T("2D Fix"); break;
    case 3:  dlg.m_mode = _T("3D Fix"); break;
    case 4:  dlg.m_mode = _T("3D Fix (SBAS)"); break;
    default: dlg.m_mode = _T("Unknown"); break;
  }
  if (pData->mode > 0 && pData->mode < 5) {
    dlg.m_time.Format(_T("%d.%02d.%02d %02d:%02d:%02d"), (pData->date >> 16), (pData->date >> 8) & 0x00FF, pData->date & 0x00FF, (pData->time >> 16), (pData->time >> 8) & 0x00FF, pData->time & 0x00FF);
    dlg.m_latitude.Format(_T("%d"), pData->lat);
    dlg.m_longitude.Format(_T("%d"), pData->lon);
    dlg.m_speed.Format(_T("%d.%d km/h"), pData->speed / 10, pData->speed % 10);
    dlg.m_direction.Format(_T("%d.%d"), pData->direction / 10, pData->direction % 10);
  } else {
    dlg.m_time      = _T("N/A");
    dlg.m_latitude  = _T("N/A");
    dlg.m_longitude = _T("N/A");
    dlg.m_speed     = _T("N/A");
    dlg.m_direction = _T("N/A");
  }
  dlg.m_supply.Format(_T("%d.%d V"), pData->supply / 10, pData->supply % 10);
  dlg.DoModal();
}

void CPageEvents::Export(CString filename)
{
  // Var
  FrameEventInfo *pData;
  CString        text;
  int            i;
  CFile          logFile(filename, CFile::modeCreate|CFile::modeWrite);
  CArchive       ar(&logFile, CArchive::store);

  // Iterate events
  for (i = 0; i < m_events.GetItemCount(); i++) {
    pData = (FrameEventInfo*)m_events.GetItemData(i);
    ar.WriteString(m_events.GetItemText(i, 1));
    ar.WriteString(_T(","));
    ar.WriteString(m_events.GetItemText(i, 2));
    ar.WriteString(_T(","));
    switch (pData->mode) {
      case 0:  ar.WriteString(_T("No info,")); break;
      case 1:  ar.WriteString(_T("No Fix,")); break;
      case 2:  ar.WriteString(_T("2D Fix,")); break;
      case 3:  ar.WriteString(_T("3D Fix,")); break;
      case 4:  ar.WriteString(_T("3D Fix (SBAS),")); break;
      default: ar.WriteString(_T("Unknown,")); break;
    }
    if (pData->mode > 0 && pData->mode < 5) {
      text.Format(_T("%d.%02d.%02d %02d:%02d:%02d,"), (pData->date >> 16), (pData->date >> 8) & 0x00FF, pData->date & 0x00FF, (pData->time >> 16), (pData->time >> 8) & 0x00FF, pData->time & 0x00FF);
      ar.WriteString(text);
      text.Format(_T("%d,"), pData->lat);
      ar.WriteString(text);
      text.Format(_T("%d,"), pData->lon);
      ar.WriteString(text);
      text.Format(_T("%d.%d km/h,"), pData->speed / 10, pData->speed % 10);
      ar.WriteString(text);
      text.Format(_T("%d.%d,"), pData->direction / 10, pData->direction % 10);
      ar.WriteString(text);
    } else {
      ar.WriteString(_T("N/A,N/A,N/A,N/A,N/A,"));
    }
    text.Format(_T("%d.%d V\r\n"), pData->supply / 10, pData->supply % 10);
    ar.WriteString(text);
  }
}

void CPageEvents::ExportKml(CString filename)
{
  // Var
  FrameEventInfo *pData;
  CString        text, temp, NodeId;
  CStringList    placement;
  int            i;
  double         lon, lat;
  CFile          logFile(filename, CFile::modeCreate|CFile::modeWrite);
  CArchive       ar(&logFile, CArchive::store);

  // Filter
  NodeId.Format(_T("%d"), AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001));

  // Header
  ar.WriteString(_T("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n"));
  ar.WriteString(_T("<kml xmlns=\"http://www.opengis.net/kml/2.2\">\r\n"));
  ar.WriteString(_T("  <Document>\r\n"));
  ar.WriteString(_T("    <name>Advanced tracking demo</name>\r\n"));
  ar.WriteString(_T("    <description>The path of all received events</description>\r\n"));
  ar.WriteString(_T("    <open>1</open>\r\n"));
  ar.WriteString(_T("    <Style id=\"yellowLineGreenPoly\">\r\n"));
  ar.WriteString(_T("      <LineStyle>\r\n"));
  ar.WriteString(_T("        <color>7f00ffff</color>\r\n"));
  ar.WriteString(_T("        <width>4</width>\r\n"));
  ar.WriteString(_T("      </LineStyle>\r\n"));
  ar.WriteString(_T("      <PolyStyle>\r\n"));
  ar.WriteString(_T("        <color>7f00ff00</color>\r\n"));
  ar.WriteString(_T("      </PolyStyle>\r\n"));
  ar.WriteString(_T("    </Style>\r\n"));
  ar.WriteString(_T("    <Placemark>\r\n"));
  ar.WriteString(_T("      <name>Tracked route</name>\r\n"));
  ar.WriteString(_T("      <styleUrl>#yellowLineGreenPoly</styleUrl>\r\n"));
  ar.WriteString(_T("      <LineString>\r\n"));
  ar.WriteString(_T("        <tessellate>0</tessellate>\r\n"));
  ar.WriteString(_T("        <coordinates>\r\n"));

  // Iterate events
  for (i = 0; i < m_events.GetItemCount(); i++) {
    if (NodeId.Compare(m_events.GetItemText(i, 1))) continue;
    pData = (FrameEventInfo*)m_events.GetItemData(i);
    if (pData->mode > 0 && pData->mode < 5) {
      // Calculate coordinate
      lat  = pData->lat % 1000000;
      lat /= 600000;
      lat += pData->lat / 1000000;
      lon  = pData->lon % 1000000;
      lon /= 600000;
      lon += pData->lon / 1000000;
      // Write coordinate
      text.Format(_T("          %f,%f,0\r\n"), lon, lat);
      ar.WriteString(text);
      // Make event placemark
      text  = _T("      <Placemark>\r\n        <name>");
      text += m_events.GetItemText(i, 2);
      text += _T("</name>\r\n        <description><![CDATA[<hr><table border=\"0\">");
      temp.Format(_T("<tr><td>Time (UTC):</td><td>%d.%02d.%02d %02d:%02d:%02d</td></tr>"), (pData->date >> 16), (pData->date >> 8) & 0x00FF, pData->date & 0x00FF, (pData->time >> 16), (pData->time >> 8) & 0x00FF, pData->time & 0x00FF);
      text += temp;
      temp.Format(_T("<tr><td>Speed:</td><td>%d.%d km/h</td></tr>"), pData->speed / 10, pData->speed % 10);
      text += temp;
      temp.Format(_T("<tr><td>Supply voltage:</td><td>%d.%d V</td></tr>"), pData->supply / 10, pData->supply % 10);
      text += temp;
      text += _T("</table><hr>]]></description>\r\n        <visibility>0</visibility>\r\n        <Point>\r\n");
      temp.Format(_T("          <coordinates>%f,%f</coordinates>\r\n"), lon, lat);
      text += temp;
      text += _T("        </Point>\r\n      </Placemark>\r\n");
      placement.AddTail(text);
    }
  }

  // Footer
  ar.WriteString(_T("        </coordinates>\r\n"));
  ar.WriteString(_T("      </LineString>\r\n"));
  ar.WriteString(_T("    </Placemark>\r\n"));
  if (!placement.IsEmpty()) {
    ar.WriteString(_T("    <Folder>\r\n"));
    ar.WriteString(_T("      <name>Placemarks</name>\r\n"));
    ar.WriteString(_T("      <description>These are the actual events at the positions</description>\r\n"));
    ar.WriteString(_T("      <open>0</open>\r\n"));
    ar.WriteString(_T("      <visibility>0</visibility>\r\n"));
    POSITION pos = placement.GetHeadPosition();
    while (pos) ar.WriteString(placement.GetNext(pos));
    ar.WriteString(_T("    </Folder>\r\n"));
  }
  ar.WriteString(_T("  </Document>\r\n"));
  ar.WriteString(_T("</kml>\r\n"));
}
/*

void CPageEvents::ExportKml(CString filename)
{
  // Var
  FrameEventInfo *pData;
  CString        text;
  int            i;
  double         pos;
  CFile          logFile(filename, CFile::modeCreate|CFile::modeWrite);
  CArchive       ar(&logFile, CArchive::store);

  // Header
  ar.WriteString(_T("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n<kml xmlns=\"http://www.opengis.net/kml/2.2\">\r\n"));

  // Iterate events
  for (i = 0; i < m_events.GetItemCount(); i++) {
    pData = (FrameEventInfo*)m_events.GetItemData(i);
    if (pData->mode > 0 && pData->mode < 5) {
      ar.WriteString(_T("  <Placemark>\r\n"));
      ar.WriteString(_T("    <name>"));
      ar.WriteString(m_events.GetItemText(i, 1));
      ar.WriteString(_T(" - "));
      ar.WriteString(m_events.GetItemText(i, 0));
      ar.WriteString(_T("</name>\r\n"));
      ar.WriteString(_T("    <description><![CDATA["));
      ar.WriteString(m_events.GetItemText(i, 2));
      ar.WriteString(_T("<br><hr><table border=\"0\">"));
      text.Format(_T("<tr><td>Time (UTC):</td><td>%d.%02d.%02d %02d:%02d:%02d</td></tr>"), (pData->date >> 16), (pData->date >> 8) & 0x00FF, pData->date & 0x00FF, (pData->time >> 16), (pData->time >> 8) & 0x00FF, pData->time & 0x00FF);
      ar.WriteString(text);
      text.Format(_T("<tr><td>Speed:</td><td>%d.%d km/h</td></tr>"), pData->speed / 10, pData->speed % 10);
      ar.WriteString(text);
      text.Format(_T("<tr><td>Supply voltage:</td><td>%d.%d V</td></tr>"), pData->supply / 10, pData->supply % 10);
      ar.WriteString(text);
      ar.WriteString(_T("</table><hr>]]></description>\r\n"));
      ar.WriteString(_T("    <Point>\r\n"));
      pos  = pData->lon % 1000000;
      pos /= 600000;
      pos += pData->lon / 1000000;
      text.Format(_T("      <coordinates>%f,"), pos);
      ar.WriteString(text);
      pos  = pData->lat % 1000000;
      pos /= 600000;
      pos += pData->lat / 1000000;
      text.Format(_T("%f</coordinates>\r\n"), pos);
      ar.WriteString(text);
      ar.WriteString(_T("    </Point>\r\n"));
      ar.WriteString(_T("  </Placemark>\r\n"));
    }
  }

  // Footer
  ar.WriteString(_T("</kml>\r\n"));
}
*/
